@@ -1,5 +1,5 @@
 import tqdm
-import ujson as json
+import json
 import pandas as pd
 import numpy as np
 import unidecode
@@ -9,6 +9,7 @@ import unidecode
 # Convert opta json files to opta.h5
 #####################################
 
+
 def jsonfiles_to_h5(jsonfiles, h5file, append=True):
 
     eventtypesdf = pd.DataFrame(eventtypes, columns=["type_id", "type_name"])
@@ -35,61 +36,62 @@ def jsonfiles_to_h5(jsonfiles, h5file, append=True):
             "files",
         ]
     }
-    for jsonfile_url in tqdm.tqdm(jsonfiles):
-        try:
-            data = extract_data(jsonfile_url)
-
-            d["games"] += [data["game"]]
-            d["players"] += data["players"]
-            d["teams"] += data["teams"]
-            d["referees"] += [data["referee"]]
-            d["teamgamestats"] += data["teamgamestats"]
-            d["playergamestats"] += data["playergamestats"]
-
-            game_id = data["game"]["game_id"]
-            key = f"events/game_{game_id}"
-            eventsdf = pd.DataFrame(data["events"])
-            eventsdf["timestamp"] = pd.to_datetime(eventsdf["timestamp"])
-            eventsdf.to_hdf(h5file, key=key)
-            d["files"] += [{"file_url": jsonfile_url, "corrupt": False}]
-        except (ValueError, MissingDataError):
-            d["files"] += [{"file_url": jsonfile_url, "corrupt": True}]
-
-    deduplic = dict(
-        games=("game_id", "game_id"),
-        teams=(["team_id", "team_name", "team_short", "team_abbr"], "team_id"),
-        players=(
-            ["player_id", "firstname", "lastname", "fullname", "knownname"],
-            "player_id",
-        ),
-        referees=(["referee_firstname", "referee_lastname"], "referee_id"),
-        teamgamestats=(["game_id", "team_id"], ["game_id", "team_id"]),
-        playergamestats=(
-            ["game_id", "team_id", "player_id"],
-            ["game_id", "team_id", "player_id"],
-        ),
-        files=("file_url", "file_url"),
-    )
-
-    for k,v in d.items():
-        d[k] = pd.DataFrame(v)
-    d["games"]["game_date"] = pd.to_datetime(d["games"]["game_date"])
-
-    for k, df in d.items():
-        if append:
+    with pd.HDFStore(h5file) as optastore:
+        for jsonfile_url in tqdm.tqdm(jsonfiles):
             try:
-                ori_df = pd.read_hdf(h5file, key=k)
-                df = pd.concat([ori_df, df])
-            except (FileNotFoundError, KeyError):
-                pass
-        sortcols, idcols = deduplic[k]
-        df.sort_values(by=sortcols, ascending=False, inplace=True)
-        df.drop_duplicates(subset=idcols, inplace=True)
-        df.to_hdf(h5file, key=k)
+                data = extract_data(jsonfile_url)
+
+                d["games"] += [data["game"]]
+                d["players"] += data["players"]
+                d["teams"] += data["teams"]
+                d["referees"] += [data["referee"]]
+                d["teamgamestats"] += data["teamgamestats"]
+                d["playergamestats"] += data["playergamestats"]
+
+                game_id = data["game"]["game_id"]
+                key = f"events/game_{game_id}"
+                eventsdf = pd.DataFrame(data["events"])
+                eventsdf["timestamp"] = pd.to_datetime(eventsdf["timestamp"])
+                optastore[key] = eventsdf
+                d["files"] += [{"file_url": jsonfile_url, "corrupt": False}]
+            except (ValueError, MissingDataError):
+                d["files"] += [{"file_url": jsonfile_url, "corrupt": True}]
+
+        deduplic = dict(
+            games=("game_id", "game_id"),
+            teams=(["team_id", "team_name", "team_short", "team_abbr"], "team_id"),
+            players=(
+                ["player_id", "firstname", "lastname", "fullname", "knownname"],
+                "player_id",
+            ),
+            referees=(["referee_firstname", "referee_lastname"], "referee_id"),
+            teamgamestats=(["game_id", "team_id"], ["game_id", "team_id"]),
+            playergamestats=(
+                ["game_id", "team_id", "player_id"],
+                ["game_id", "team_id", "player_id"],
+            ),
+            files=("file_url", "file_url"),
+        )
+
+        for k, v in d.items():
+            d[k] = pd.DataFrame(v)
+        d["games"]["game_date"] = pd.to_datetime(d["games"]["game_date"])
+
+        for k, df in d.items():
+            if append:
+                try:
+                    ori_df = optastore[k]
+                    df = pd.concat([ori_df, df])
+                except (FileNotFoundError, KeyError):
+                    pass
+            sortcols, idcols = deduplic[k]
+            df.sort_values(by=sortcols, ascending=False, inplace=True)
+            df.drop_duplicates(subset=idcols, inplace=True)
+            optastore[k] = df
 
 
 def extract_data(jsonfile):
-    with open(jsonfile) as fh:
+    with open(jsonfile, encoding="utf-8") as fh:
         root = json.load(fh)
 
     return {
@@ -165,7 +167,7 @@ def extract_game(root):
         venue_id=int(venue["@attributes"]["uID"].replace("v", "")),
         referee_id=int(matchofficial["@attributes"]["uID"].replace("o", "")),
         game_date=pd.to_datetime(assertget(matchinfo, "Date")),
-        attendance=int(matchinfo.get("Attendance",0)),
+        attendance=int(matchinfo.get("Attendance", 0)),
         duration=int(stat["@value"]),
     )
     return game_dict
@@ -466,73 +468,72 @@ import socceraction.spadl.config as spadlcfg
 spadl_length = spadlcfg.spadl_length
 spadl_width = spadlcfg.spadl_width
 
-bodyparts =  spadlcfg.bodyparts
+bodyparts = spadlcfg.bodyparts
 results = spadlcfg.results
 actiontypes = spadlcfg.actiontypes
 
+
 def convert_to_spadl(optah5, spadlh5):
 
-    games = pd.read_hdf(optah5, key="games")
-    games.to_hdf(spadlh5, key="games")
+    with pd.HDFStore(optah5) as optastore, pd.HDFStore(spadlh5) as spadlstore:
+        games = optastore["games"]
+        spadlstore["games"] = games
 
-    players = pd.read_hdf(optah5, key="players")
-    players = players.rename(
-        index=str,
-        columns={
-            "firstname": "first_name",
-            "lastname": "last_name",
-            "knownname": "soccer_name",
-        },
-    )
-    players["birthday"] = pd.NaT  # unavailabe
-    players["nation_id"] = np.nan  # unavailable
-    players.to_hdf(spadlh5, key="players")
-
-    teams = pd.read_hdf(optah5, key="teams")
-    teams.to_hdf(spadlh5, key="teams")
-
-    teamgames = pd.read_hdf(optah5, key="teamgamestats")
-    teamgames = teamgames.rename(index=str, columns={"formation_used": "formation"})
-    teamgames.to_hdf(spadlh5, key="team_games")
-
-    playergames = pd.read_hdf(optah5, key="playergamestats")
-    playergames = teamgames.rename(
-        index=str,
-        columns={
-            "mins_played": "minutes_played",
-            "goal_assists": "assists",
-            "total_att_assist": "keypasses",
-            "second_goal_assist": "pre_assists",
-        },
-    )
-    playergames.to_hdf(spadlh5, key="player_games")
-
-    actiontypesdf = pd.DataFrame(
-        list(enumerate(actiontypes)), columns=["type_id", "type_name"]
-    )
-    actiontypesdf.to_hdf(spadlh5, key="actiontypes")
-
-    bodypartsdf = pd.DataFrame(
-        list(enumerate(bodyparts)), columns=["bodypart_id", "bodypart_name"]
-    )
-    bodypartsdf.to_hdf(spadlh5, key="bodyparts")
-
-    resultsdf = pd.DataFrame(
-        list(enumerate(results)), columns=["result_id", "result_name"]
-    )
-    resultsdf.to_hdf(spadlh5, key="results")
-
-    eventtypes = pd.read_hdf(optah5, "eventtypes")
-    for game in tqdm.tqdm(list(games.itertuples())):
-
-        events = pd.read_hdf(optah5, f"events/game_{game.game_id}")
-        events = (
-            events.merge(eventtypes, on="type_id")
-            .sort_values(["game_id", "period_id", "minute", "second","timestamp"])
-            .reset_index(drop=True)
+        players = optastore["players"]
+        players = players.rename(
+            index=str,
+            columns={
+                "firstname": "first_name",
+                "lastname": "last_name",
+                "knownname": "soccer_name",
+            },
         )
-        actions = convert_to_actions(events, home_team_id=game.home_team_id)
-        actions.to_hdf(spadlh5, f"actions/game_{game.game_id}")
+        # players["birthday"] = pd.NaT  # unavailabe
+        # players["nation_id"] = np.nan  # unavailable
+        spadlstore["players"] = players
+
+        teams = optastore["teams"]
+        spadlstore["teams"] = teams
+
+        teamgames = optastore["teamgamestats"]
+        teamgames = teamgames.rename(index=str, columns={"formation_used": "formation"})
+        spadlstore["teamgames"] = teamgames
+
+        playergames = optastore["playergamestats"]
+        playergames = playergames.rename(
+            index=str,
+            columns={
+                "mins_played": "minutes_played",
+                "goal_assists": "assists",
+                "total_att_assist": "keypasses",
+                "second_goal_assist": "pre_assists",
+            },
+        )
+        spadlstore["playergames"] = playergames
+
+        spadlstore["actiontypes"] = pd.DataFrame(
+            list(enumerate(actiontypes)), columns=["type_id", "type_name"]
+        )
+
+        spadlstore["bodyparts"] = pd.DataFrame(
+            list(enumerate(bodyparts)), columns=["bodypart_id", "bodypart_name"]
+        )
+
+        spadlstore["results"] = pd.DataFrame(
+            list(enumerate(results)), columns=["result_id", "result_name"]
+        )
+
+        eventtypes = optastore["eventtypes"]
+        for game in tqdm.tqdm(list(games.itertuples())):
+
+            events = optastore[f"events/game_{game.game_id}"]
+            events = (
+                events.merge(eventtypes, on="type_id", how="left")
+                .sort_values(["game_id", "period_id", "minute", "second", "timestamp"])
+                .reset_index(drop=True)
+            )
+            actions = convert_to_actions(events, home_team_id=game.home_team_id)
+            spadlstore[f"actions/game_{game.game_id}"] = actions
 
 
 def convert_to_actions(events, home_team_id):
@@ -593,7 +594,7 @@ def get_result_id(args):
     if e == "offside pass":
         r = "offside"  # offside
     elif e == "foul":
-        r = "success"
+        r = "fail"
     elif e in ["attempt saved", "miss", "post"]:
         r = "fail"
     elif e == "goal":
@@ -601,7 +602,7 @@ def get_result_id(args):
             r = "owngoal"  # own goal, x and y must be switched
         else:
             r = "success"
-    elif e == "bad touch":
+    elif e == "ball touch":
         r = "fail"
     elif outcome:
         r = "success"
@@ -685,7 +686,7 @@ def add_dribbles(actions):
     next_actions = actions.shift(-1)
 
     same_team = actions.team_id == next_actions.team_id
-    #not_clearance = actions.type_id != actiontypes.index("clearance")
+    # not_clearance = actions.type_id != actiontypes.index("clearance")
 
     dx = actions.end_x - next_actions.start_x
     dy = actions.end_y - next_actions.start_y
@@ -719,13 +720,16 @@ def add_dribbles(actions):
     actions.reset_index(drop=True, inplace=True)
     return actions
 
+
 def fix_clearances(actions):
     next_actions = actions.shift(-1)
+    next_actions[-1:] = actions[-1:]
     clearance_idx = actions.type_id == actiontypes.index("clearance")
-    actions.loc[clearance_idx,"end_x"] = next_actions[clearance_idx].start_x.values
-    actions.loc[clearance_idx,"end_y"] = next_actions[clearance_idx].start_y.values
+    actions.loc[clearance_idx, "end_x"] = next_actions[clearance_idx].start_x.values
+    actions.loc[clearance_idx, "end_y"] = next_actions[clearance_idx].start_y.values
     return actions
 
+
 def fix_direction_of_play(actions, home_team_id):
     away_idx = (actions.team_id != home_team_id).values
     for col in ["start_x", "end_x"]:
@@ -733,4 +737,4 @@ def fix_direction_of_play(actions, home_team_id):
     for col in ["start_y", "end_y"]:
         actions.loc[away_idx, col] = spadl_width - actions[away_idx][col].values
 
-    return actions
\ No newline at end of file
+    return actions
